@@ -6,6 +6,7 @@ from .. import util
 from ..dimension import Dimension, dimension_name
 from ..element import Element
 from ..ndmapping import NdMapping, item_check, sorted_context
+from ..util import dtype_kind
 from ..util.dependencies import PANDAS_GE_2_1_0, _LazyModule
 from .interface import DataError, Interface
 from .util import finite_range
@@ -44,7 +45,7 @@ class PandasInterface(Interface, PandasAPI):
     def applies(cls, obj):
         if not cls.loaded():
             return False
-        return type(obj) is pd.DataFrame
+        return type(obj) is pd.DataFrame or type(obj) is pd.Series
 
     @classmethod
     def dimension_type(cls, dataset, dim):
@@ -209,7 +210,7 @@ class PandasInterface(Interface, PandasAPI):
             column = cls.index_values(dataset, dimension)
         else:
             column = dataset.data[dimension.name]
-        if column.dtype.kind == 'O':
+        if dtype_kind(column) == 'O':
             if not isinstance(dataset.data, pd.DataFrame):
                 column = column.sort(inplace=False)
             else:
@@ -227,7 +228,7 @@ class PandasInterface(Interface, PandasAPI):
             if dimension.nodata is not None:
                 column = cls.replace_value(column, dimension.nodata)
             cmin, cmax = finite_range(column, column.min(), column.max())
-            if column.dtype.kind == 'M' and getattr(column.dtype, 'tz', None):
+            if dtype_kind(column) == 'M' and getattr(column.dtype, 'tz', None):
                 return (cmin.to_pydatetime().replace(tzinfo=None),
                         cmax.to_pydatetime().replace(tzinfo=None))
             return cmin, cmax
@@ -455,7 +456,7 @@ class PandasInterface(Interface, PandasAPI):
             data = dataset.data[dim.name]
         if keep_index:
             return data
-        if data.dtype.kind == 'M' and getattr(data.dtype, 'tz', None):
+        if dtype_kind(data) == 'M' and getattr(data.dtype, 'tz', None):
             data = (data if isindex else data.dt).tz_localize(None)
         if not expanded:
             return pd.unique(data)
