@@ -123,10 +123,10 @@ def list_formats(format_type, backend=None):
     """
     if backend is None:
         backend = Store.current_backend
-        mode = Store.renderers[backend].mode if backend in Store.renderers else None
+        _mode = Store.renderers[backend].mode if backend in Store.renderers else None
     else:
         split = backend.split(':')
-        backend, mode = split if len(split)==2 else (split[0], 'default')
+        backend, _mode = split if len(split)==2 else (split[0], 'default')
 
     if backend in Store.renderers:
         return Store.renderers[backend].mode_formats[format_type]
